@@ -1,21 +1,35 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#   (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP. If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 import numpy as np
 from qutip.qobj import Qobj
 from qutip.expect import expect
@@ -25,7 +39,7 @@ from qutip.operators import sigmax, sigmay, sigmaz
 class Bloch3d():
     """Class for plotting data on a 3D Bloch sphere using mayavi.
     Valid data can be either points, vectors, or qobj objects
-    corresponding to state vectors or density matricies for
+    corresponding to state vectors or density matrices. for
     a two-state system (or subsystem).
 
     Attributes
@@ -91,13 +105,13 @@ class Bloch3d():
 
     """
     def __init__(self, fig=None):
-        #----check for mayavi-----
+        # ----check for mayavi-----
         try:
             from mayavi import mlab
         except:
             raise Exception("This function requires the mayavi module.")
 
-        #---Image options---
+        # ---Image options---
         self.fig = None
         self.user_fig = None
         # check if user specified figure or axes.
@@ -112,13 +126,13 @@ class Bloch3d():
         # Image foreground color. Other options can override.
         self.fgcolor = 'black'
 
-        #---Sphere options---
+        # ---Sphere options---
         # Color of Bloch sphere, default = #808080
         self.sphere_color = '#808080'
         # Transparency of Bloch sphere, default = 0.1
         self.sphere_alpha = 0.1
 
-        #---Frame options---
+        # ---Frame options---
         # Draw frame?
         self.frame = True
         # number of lines to draw for frame
@@ -130,7 +144,7 @@ class Bloch3d():
         # Radius of frame lines
         self.frame_radius = 0.005
 
-        #--Axes---
+        # --Axes---
         # Axes color
         self.axes_color = 'black'
         # Transparency of axes
@@ -138,7 +152,7 @@ class Bloch3d():
         # Radius of axes lines
         self.axes_radius = 0.005
 
-        #---Labels---
+        # ---Labels---
         # Labels for x-axis (in LaTex), default = ['$x$','']
         self.xlabel = ['|x>', '']
         # Position of x-axis labels, default = [1.2,-1.2]
@@ -152,13 +166,13 @@ class Bloch3d():
         # Position of z-axis labels, default = [1.05,-1.05]
         self.zlpos = [1.07, -1.07]
 
-        #---Font options---
+        # ---Font options---
         # Color of fonts, default = 'black'
         self.font_color = 'black'
         # Size of fonts, default = 20
         self.font_scale = 0.08
 
-        #---Vector options---
+        # ---Vector options---
         # Object used for representing vectors on Bloch sphere.
         # List of colors for Bloch vectors, default = ['b','g','r','y']
         self.vector_color = ['r', 'g', 'b', 'y']
@@ -171,7 +185,7 @@ class Bloch3d():
         # Radius of vector head
         self.vector_head_radius = 0.075
 
-        #---Point options---
+        # ---Point options---
         # List of colors for Bloch point markers, default = ['b','g','r','y']
         self.point_color = ['r', 'g', 'b', 'y']
         # Size of point markers
@@ -181,15 +195,11 @@ class Bloch3d():
         # Default = 'sphere'
         self.point_mode = 'sphere'
 
-        #---Data lists---
+        # ---Data lists---
         # Data for point markers
         self.points = []
-        # Number of point markers to plot
-        self.num_points = 0
         # Data for Bloch vectors
         self.vectors = []
-        # Number of Bloch vectors to plot
-        self.num_vectors = 0
         # Number of times sphere has been saved
         self.savenum = 0
         # Style of points, 'm' for multiple colors, 's' for single color
@@ -199,8 +209,8 @@ class Bloch3d():
         s = ""
         s += "Bloch3D data:\n"
         s += "-----------\n"
-        s += "Number of points:  " + str(self.num_points) + "\n"
-        s += "Number of vectors: " + str(self.num_vectors) + "\n"
+        s += "Number of points:  " + str(len(self.points)) + "\n"
+        s += "Number of vectors: " + str(len(self.vectors)) + "\n"
         s += "\n"
         s += "Bloch3D sphere properties:\n"
         s += "--------------------------\n"
@@ -240,9 +250,7 @@ class Bloch3d():
         """Resets the Bloch sphere data sets to empty.
         """
         self.points = []
-        self.num_points = 0
         self.vectors = []
-        self.num_vectors = 0
         self.point_style = []
 
     def add_points(self, points, meth='s'):
@@ -268,11 +276,9 @@ class Bloch3d():
             else:
                 pnts = points
             self.points.append(pnts)
-            self.num_points = len(self.points)
             self.point_style.append('s')
         else:
             self.points.append(points)
-            self.num_points = len(self.points)
             self.point_style.append('m')
 
     def add_states(self, state, kind='vector'):
@@ -311,10 +317,8 @@ class Bloch3d():
         if isinstance(vectors[0], (list, np.ndarray)):
             for vec in vectors:
                 self.vectors.append(vec)
-                self.num_vectors = len(self.vectors)
         else:
             self.vectors.append(vectors)
-            self.num_vectors = len(self.vectors)
 
     def plot_vectors(self):
         """
@@ -324,35 +328,34 @@ class Bloch3d():
         from tvtk.api import tvtk
         import matplotlib.colors as colors
         ii = 0
-        if len(self.vectors) > 0:
-            for k in range(len(self.vectors)):
-                vec = np.array(self.vectors[k])
-                norm = np.linalg.norm(vec)
-                vec = vec / norm
-                theta = np.arccos(vec[2])
-                phi = np.arctan2(vec[1], vec[0])
-                vec = vec * norm
-                vec2 = vec / np.sqrt((norm + self.vector_head_height))
-                color = colors.colorConverter.to_rgb(
-                    self.vector_color[np.mod(k, len(self.vector_color))])
-                mlab.plot3d([0, vec2[0]], [0, vec2[1]], [0, vec2[2]],
-                            name='vector' + str(ii), tube_sides=100,
-                            line_width=self.vector_width,
-                            opacity=self.vector_alpha,
-                            color=color)
-                cone = tvtk.ConeSource(height=self.vector_head_height,
-                                       radius=self.vector_head_radius,
-                                       resolution=100)
-                cone_mapper = tvtk.PolyDataMapper(input=cone.output)
-                color = colors.colorConverter.to_rgb(
-                    self.vector_color[np.mod(k, len(self.vector_color))])
-                prop = tvtk.Property(opacity=self.vector_alpha, color=color)
-                cc = tvtk.Actor(mapper=cone_mapper, property=prop)
-                cc.rotate_z(np.degrees(phi))
-                cc.rotate_y(-90 + np.degrees(theta))
-                cc.position = np.array([vec[0], vec[1], vec[2]]) / \
-                    np.sqrt((norm + self.vector_head_height))
-                self.fig.scene.add_actor(cc)
+        for k in range(len(self.vectors)):
+            vec = np.array(self.vectors[k])
+            norm = np.linalg.norm(vec)
+            theta = np.arccos(vec[2] / norm)
+            phi = np.arctan2(vec[1], vec[0])
+            vec -= 0.5 * self.vector_head_height * \
+                np.array([np.sin(theta) * np.cos(phi),
+                          np.sin(theta) * np.sin(phi), np.cos(theta)])
+
+            color = colors.colorConverter.to_rgb(
+                self.vector_color[np.mod(k, len(self.vector_color))])
+
+            mlab.plot3d([0, vec[0]], [0, vec[1]], [0, vec[2]],
+                        name='vector' + str(ii), tube_sides=100,
+                        line_width=self.vector_width,
+                        opacity=self.vector_alpha,
+                        color=color)
+
+            cone = tvtk.ConeSource(height=self.vector_head_height,
+                                   radius=self.vector_head_radius,
+                                   resolution=100)
+            cone_mapper = tvtk.PolyDataMapper(input=cone.output)
+            prop = tvtk.Property(opacity=self.vector_alpha, color=color)
+            cc = tvtk.Actor(mapper=cone_mapper, property=prop)
+            cc.rotate_z(np.degrees(phi))
+            cc.rotate_y(-90 + np.degrees(theta))
+            cc.position = vec
+            self.fig.scene.add_actor(cc)
 
     def plot_points(self):
         """
@@ -360,43 +363,41 @@ class Bloch3d():
         """
         from mayavi import mlab
         import matplotlib.colors as colors
-        if self.num_points > 0:
-            for k in range(self.num_points):
-                num = len(self.points[k][0])
-                dist = [np.sqrt(self.points[k][0][j] ** 2 +
-                                self.points[k][1][j] ** 2 +
-                                self.points[k][2][j] ** 2) for j in range(num)]
-                if any(abs(dist - dist[0]) / dist[0] > 1e-12):
-                    # combine arrays so that they can be sorted together
-                    zipped = zip(dist, range(num))
-                    zipped.sort()  # sort rates from lowest to highest
-                    dist, indperm = zip(*zipped)
-                    indperm = np.array(indperm)
-                else:
-                    indperm = range(num)
-                if self.point_style[k] == 's':
-                    color = colors.colorConverter.to_rgb(
-                        self.point_color[np.mod(k, len(self.point_color))])
-                    mlab.points3d(
-                        self.points[k][0][indperm], self.points[k][1][indperm],
-                        self.points[k][2][
-                            indperm], figure=self.fig, resolution=100,
-                        scale_factor=self.point_size, mode=self.point_mode,
-                        color=color)
+        for k in range(len(self.points)):
+            num = len(self.points[k][0])
+            dist = [np.sqrt(self.points[k][0][j] ** 2 +
+                            self.points[k][1][j] ** 2 +
+                            self.points[k][2][j] ** 2) for j in range(num)]
+            if any(abs(dist - dist[0]) / dist[0] > 1e-12):
+                # combine arrays so that they can be sorted together
+                zipped = zip(dist, range(num))
+                zipped.sort()  # sort rates from lowest to highest
+                dist, indperm = zip(*zipped)
+                indperm = np.array(indperm)
+            else:
+                indperm = range(num)
+            if self.point_style[k] == 's':
+                color = colors.colorConverter.to_rgb(
+                    self.point_color[np.mod(k, len(self.point_color))])
+                mlab.points3d(
+                    self.points[k][0][indperm], self.points[k][1][indperm],
+                    self.points[k][2][indperm], figure=self.fig,
+                    resolution=100, scale_factor=self.point_size,
+                    mode=self.point_mode, color=color)
 
-                elif self.point_style[k] == 'm':
-                    pnt_colors = np.array(self.point_color * np.ceil(
-                        num / float(len(self.point_color))))
-                    pnt_colors = pnt_colors[0:num]
-                    pnt_colors = list(pnt_colors[indperm])
-                    for kk in range(num):
-                        mlab.points3d(
-                            self.points[k][0][
-                                indperm[kk]], self.points[k][1][indperm[kk]],
-                            self.points[k][2][
-                                indperm[kk]], figure=self.fig, resolution=100,
-                            scale_factor=self.point_size, mode=self.point_mode,
-                            color=colors.colorConverter.to_rgb(pnt_colors[kk]))
+            elif self.point_style[k] == 'm':
+                pnt_colors = np.array(self.point_color * np.ceil(
+                    num / float(len(self.point_color))))
+                pnt_colors = pnt_colors[0:num]
+                pnt_colors = list(pnt_colors[indperm])
+                for kk in range(num):
+                    mlab.points3d(
+                        self.points[k][0][
+                            indperm[kk]], self.points[k][1][indperm[kk]],
+                        self.points[k][2][
+                            indperm[kk]], figure=self.fig, resolution=100,
+                        scale_factor=self.point_size, mode=self.point_mode,
+                        color=colors.colorConverter.to_rgb(pnt_colors[kk]))
 
     def make_sphere(self):
         """
@@ -434,16 +435,14 @@ class Bloch3d():
                 xlon = np.sin(angle) * np.sin(theta)
                 ylon = np.cos(angle) * np.sin(theta)
                 zlon = np.cos(theta)
-                mlab.plot3d(xlat, ylat, zlat,
-                            color=colors.colorConverter.to_rgb(
-                            self.frame_color),
-                            opacity=self.frame_alpha,
-                            tube_radius=self.frame_radius)
-                mlab.plot3d(xlon, ylon, zlon,
-                            color=colors.colorConverter.to_rgb(
-                            self.frame_color),
-                            opacity=self.frame_alpha,
-                            tube_radius=self.frame_radius)
+                mlab.plot3d(
+                    xlat, ylat, zlat,
+                    color=colors.colorConverter.to_rgb(self.frame_color),
+                    opacity=self.frame_alpha, tube_radius=self.frame_radius)
+                mlab.plot3d(
+                    xlon, ylon, zlon,
+                    color=colors.colorConverter.to_rgb(self.frame_color),
+                    opacity=self.frame_alpha, tube_radius=self.frame_radius)
 
         # add axes
         axis = np.linspace(-1.0, 1.0, 10)
