@@ -1,21 +1,35 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 
 import numpy as np
 import scipy.linalg as la
@@ -25,18 +39,18 @@ from types import FunctionType
 from qutip.qobj import Qobj, isket
 from qutip.superoperator import vec2mat_index, mat2vec, vec2mat
 from qutip.mesolve import mesolve
-from qutip.steady import steadystate
+from qutip.steadystate import steadystate
 from qutip.states import ket2dm
 from qutip.states import projection
-from qutip.odeoptions import Odeoptions
+from qutip.solver import Options
 from qutip.propagator import propagator
-from qutip.odedata import Odedata
-from qutip.cyQ.spmatfuncs import cy_ode_rhs
+from qutip.solver import Result
+from qutip.cy.spmatfuncs import cy_ode_rhs
 from qutip.expect import expect
 from qutip.utilities import n_thermal
 
 
-def floquet_modes(H, T, args=None, sort=False):
+def floquet_modes(H, T, args=None, sort=False, U=None):
     """
     Calculate the initial Floquet modes Phi_alpha(0) for a driven system with
     period T.
@@ -59,6 +73,11 @@ def floquet_modes(H, T, args=None, sort=False):
         The period of the time-dependence of the hamiltonian. The default value
         'None' indicates that the 'tlist' spans a single period of the driving.
 
+    U : :class:`qutip.qobj`
+        The propagator for the time-dependent Hamiltonian with period `T`.
+        If U is `None` (default), it will be calculated from the Hamiltonian
+        `H` using :func:`qutip.propagator.propagator`.
+
     Returns
     -------
 
@@ -68,8 +87,9 @@ def floquet_modes(H, T, args=None, sort=False):
 
     """
 
-    # get the unitary propagator
-    U = propagator(H, T, [], args)
+    if U is None:
+        # get the unitary propagator
+        U = propagator(H, T, [], args)
 
     # find the eigenstates for the propagator
     evals, evecs = la.eig(U.full())
@@ -192,7 +212,7 @@ def floquet_modes_table(f_modes_0, f_energies, tlist, H, T, args=None):
 
     f_modes_table_t = [[] for t in tlist_period]
 
-    opt = Odeoptions()
+    opt = Options()
     opt.rhs_reuse = True
 
     for n, f_mode in enumerate(f_modes_0):
@@ -452,9 +472,9 @@ def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
     Returns
     -------
 
-    output : :class:`qutip.odedata.Odedata`
+    output : :class:`qutip.solver.Result`
 
-        An instance of the class :class:`qutip.odedata.Odedata`, which
+        An instance of the class :class:`qutip.solver.Result`, which
         contains either an *array* of expectation values or an array of
         state vectors, for the times specified by `tlist`.
     """
@@ -471,8 +491,8 @@ def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
                                           np.linspace(0, T, Tsteps + 1),
                                           H, T, args)
 
-    # setup Odedata for storing the results
-    output = Odedata()
+    # setup Result for storing the results
+    output = Result()
     output.times = tlist
     output.solver = "fsesolve"
 
@@ -498,11 +518,11 @@ def fsesolve(H, psi0, tlist, e_ops=[], T=None, args={}, Tsteps=100):
     else:
         raise TypeError("e_ops must be a list Qobj or a callback function")
 
-    psi0_fb = psi0.transform(f_modes_0, True)
+    psi0_fb = psi0.transform(f_modes_0)
     for t_idx, t in enumerate(tlist):
         f_modes_t = floquet_modes_t_lookup(f_modes_table_t, t, T)
         f_states_t = floquet_states(f_modes_t, f_energies, t)
-        psi_t = psi0_fb.transform(f_states_t, False)
+        psi_t = psi0_fb.transform(f_states_t, True)
 
         if expt_callback:
             # use callback method
@@ -735,7 +755,7 @@ def floquet_basis_transform(f_modes, f_energies, rho0):
     return rho0.transform(f_modes, True)
 
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Floquet-Markov master equation
 #
 #
@@ -746,7 +766,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
     """
 
     if options is None:
-        opt = Odeoptions()
+        opt = Options()
     else:
         opt = options
 
@@ -766,7 +786,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
     n_tsteps = len(tlist)
     dt = tlist[1] - tlist[0]
 
-    output = Odedata()
+    output = Result()
     output.solver = "fmmesolve"
     output.times = tlist
 
@@ -802,7 +822,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
     # computational basis to the floquet basis
     #
     if ekets is not None:
-        rho0 = rho0.transform(ekets, True)
+        rho0 = rho0.transform(ekets)
 
     #
     # setup integrator
@@ -833,7 +853,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
             else:
                 f_modes_table_t, T = f_modes_table
                 f_modes_t = floquet_modes_t_lookup(f_modes_table_t, t, T)
-                e_ops(t, Qobj(rho).transform(f_modes_t, False))
+                e_ops(t, Qobj(rho).transform(f_modes_t, True))
         else:
             # calculate all the expectation values, or output rho if
             # no operators
@@ -843,7 +863,7 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
                 else:
                     f_modes_table_t, T = f_modes_table
                     f_modes_t = floquet_modes_t_lookup(f_modes_table_t, t, T)
-                    output.states.append(Qobj(rho).transform(f_modes_t, False))
+                    output.states.append(Qobj(rho).transform(f_modes_t, True))
             else:
                 f_modes_table_t, T = f_modes_table
                 f_modes_t = floquet_modes_t_lookup(f_modes_table_t, t, T)
@@ -857,12 +877,12 @@ def floquet_markov_mesolve(R, ekets, rho0, tlist, e_ops, f_modes_table=None,
     return output
 
 
-#------------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Solve the Floquet-Markov master equation
 #
 #
 def fmmesolve(H, rho0, tlist, c_ops, e_ops=[], spectra_cb=[], T=None,
-              args={}, options=Odeoptions(), floquet_basis=True, kmax=5):
+              args={}, options=Options(), floquet_basis=True, kmax=5):
     """
     Solve the dynamics for the system using the Floquet-Markov master equation.
 
@@ -911,7 +931,7 @@ def fmmesolve(H, rho0, tlist, c_ops, e_ops=[], spectra_cb=[], T=None,
         >>> kB = 1.38e-23
         >>> args['w_th'] = temperature * (kB / h) * 2 * pi * 1e-9
 
-    options : :class:`qutip.odeoptions`
+    options : :class:`qutip.solver`
         options for the ODE solver.
 
     k_max : int
@@ -920,9 +940,9 @@ def fmmesolve(H, rho0, tlist, c_ops, e_ops=[], spectra_cb=[], T=None,
     Returns
     -------
 
-    output : :class:`qutip.odedata`
+    output : :class:`qutip.solver`
 
-        An instance of the class :class:`qutip.odedata`, which contains either
+        An instance of the class :class:`qutip.solver`, which contains either
         an *array* of expectation values for the times specified by `tlist`.
     """
 
