@@ -1,20 +1,34 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ###############################################################################
 """
 This module provides solvers for the unitary Schrodinger equation.
@@ -22,6 +36,7 @@ This module provides solvers for the unitary Schrodinger equation.
 
 import os
 import types
+from functools import partial
 import numpy as np
 import scipy.sparse as sp
 import scipy.integrate
@@ -30,19 +45,22 @@ from scipy.linalg import norm
 from qutip.qobj import Qobj, isket
 from qutip.expect import expect
 from qutip.rhs_generate import rhs_generate
-from qutip.odedata import Odedata
-from qutip.odeoptions import Odeoptions
-from qutip.odeconfig import odeconfig
-from qutip.odechecks import _ode_checks
+from qutip.solver import Result, Options, config
+from qutip.rhs_generate import _td_format_check, _td_wrap_array_str
 from qutip.settings import debug
-from qutip.cyQ.spmatfuncs import cy_ode_rhs
-from qutip.cyQ.codegen import Codegen
+from qutip.cy.spmatfuncs import (cy_expect_psi, cy_ode_rhs,
+                                 cy_ode_psi_func_td,
+                                 cy_ode_psi_func_td_with_state)
+from qutip.cy.codegen import Codegen
+
+from qutip.ui.progressbar import BaseProgressBar
 
 if debug:
     import inspect
 
 
-def sesolve(H, rho0, tlist, expt_ops, args={}, options=None):
+def sesolve(H, rho0, tlist, e_ops, args={}, options=None,
+            progress_bar=BaseProgressBar()):
     """
     Schrodinger equation evolution of a state vector for a given Hamiltonian.
 
@@ -52,7 +70,7 @@ def sesolve(H, rho0, tlist, expt_ops, args={}, options=None):
 
     The output is either the state vector at arbitrary points in time
     (`tlist`), or the expectation values of the supplied operators
-    (`expt_ops`). If expt_ops is a callback function, it is invoked for each
+    (`e_ops`). If e_ops is a callback function, it is invoked for each
     time in `tlist` with time and the state as arguments, and the function
     does not use any return values.
 
@@ -69,7 +87,7 @@ def sesolve(H, rho0, tlist, expt_ops, args={}, options=None):
     tlist : *list* / *array*
         list of times for :math:`t`.
 
-    expt_ops : list of :class:`qutip.qobj` / callback function single
+    e_ops : list of :class:`qutip.qobj` / callback function single
         single operator or list of operators for which to evaluate
         expectation values.
 
@@ -83,44 +101,69 @@ def sesolve(H, rho0, tlist, expt_ops, args={}, options=None):
     Returns
     -------
 
-    output: :class:`qutip.odedata`
+    output: :class:`qutip.solver`
 
-        An instance of the class :class:`qutip.odedata`, which contains either
+        An instance of the class :class:`qutip.solver`, which contains either
         an *array* of expectation values for the times specified by `tlist`, or
         an *array* or state vectors or density matrices corresponding to the
-        times in `tlist` [if `expt_ops` is an empty list], or
+        times in `tlist` [if `e_ops` is an empty list], or
         nothing if a callback function was given inplace of operators for
         which to calculate the expectation values.
 
     """
 
-    if isinstance(expt_ops, Qobj):
-        expt_ops = [expt_ops]
+    if isinstance(e_ops, Qobj):
+        e_ops = [e_ops]
+
+    if isinstance(e_ops, dict):
+        e_ops_dict = e_ops
+        e_ops = [e for e in e_ops.values()]
+    else:
+        e_ops_dict = None
+
+    # convert array based time-dependence to string format
+    H, _, args = _td_wrap_array_str(H, [], args, tlist)
 
     # check for type (if any) of time-dependent inputs
-    n_const, n_func, n_str = _ode_checks(H, [])
+    n_const, n_func, n_str = _td_format_check(H, [])
 
     if options is None:
-        options = Odeoptions()
+        options = Options()
 
-    if (not options.rhs_reuse) or (not odeconfig.tdfunc):
-        # reset odeconfig time-dependence flags to default values
-        odeconfig.reset()
+    if (not options.rhs_reuse) or (not config.tdfunc):
+        # reset config time-dependence flags to default values
+        config.reset()
 
     if n_func > 0:
-        return _sesolve_list_func_td(H, rho0, tlist, expt_ops, args, options)
+        res = _sesolve_list_func_td(H, rho0, tlist, e_ops, args, options,
+                                    progress_bar)
+
     elif n_str > 0:
-        return _sesolve_list_str_td(H, rho0, tlist, expt_ops, args, options)
-    elif isinstance(H, types.FunctionType):
-        return _sesolve_func_td(H, rho0, tlist, expt_ops, args, options)
+        res = _sesolve_list_str_td(H, rho0, tlist, e_ops, args, options,
+                                   progress_bar)
+
+    elif isinstance(H, (types.FunctionType,
+                        types.BuiltinFunctionType,
+                        partial)):
+        res = _sesolve_func_td(H, rho0, tlist, e_ops, args, options,
+                               progress_bar)
+
     else:
-        return _sesolve_const(H, rho0, tlist, expt_ops, args, options)
+        res = _sesolve_const(H, rho0, tlist, e_ops, args, options,
+                             progress_bar)
+
+    if e_ops_dict:
+        res.expect = {e: res.expect[n]
+                      for n, e in enumerate(e_ops_dict.keys())}
+
+    return res
 
 
 # -----------------------------------------------------------------------------
 # A time-dependent unitary wavefunction equation on the list-function format
 #
-def _sesolve_list_func_td(H_list, psi0, tlist, expt_ops, args, opt):
+def _sesolve_list_func_td(H_list, psi0, tlist, e_ops, args, opt,
+                          progress_bar):
     """
     Internal function for solving the master equation. See mesolve for usage.
     """
@@ -138,7 +181,10 @@ def _sesolve_list_func_td(H_list, psi0, tlist, expt_ops, args, opt):
     # construct liouvillian in list-function format
     #
     L_list = []
-    constant_func = lambda x, y: 1.0
+    if not opt.rhs_with_state:
+        constant_func = lambda x, y: 1.0
+    else:
+        constant_func = lambda x, y, z: 1.0
 
     # add all hamitonian terms to the lagrangian list
     for h_spec in H_list:
@@ -162,8 +208,11 @@ def _sesolve_list_func_td(H_list, psi0, tlist, expt_ops, args, opt):
     #
     # setup integrator
     #
-    initial_vector = psi0.full()
-    r = scipy.integrate.ode(psi_list_td)
+    initial_vector = psi0.full().ravel()
+    if not opt.rhs_with_state:
+        r = scipy.integrate.ode(psi_list_td)
+    else:
+        r = scipy.integrate.ode(psi_list_td_with_state)
     r.set_integrator('zvode', method=opt.method, order=opt.order,
                      atol=opt.atol, rtol=opt.rtol, nsteps=opt.nsteps,
                      first_step=opt.first_step, min_step=opt.min_step,
@@ -174,7 +223,8 @@ def _sesolve_list_func_td(H_list, psi0, tlist, expt_ops, args, opt):
     #
     # call generic ODE code
     #
-    return _generic_ode_solve(r, psi0, tlist, expt_ops, opt)
+    return _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar, norm,
+                              dims=psi0.dims)
 
 
 #
@@ -197,14 +247,32 @@ def psi_list_td(t, psi, H_list_and_args):
     return H * psi
 
 
+def psi_list_td_with_state(t, psi, H_list_and_args):
+
+    H_list = H_list_and_args[0]
+    args = H_list_and_args[1]
+
+    H = H_list[0][0] * H_list[0][1](t, psi, args)
+    for n in range(1, len(H_list)):
+        #
+        # args[n][0] = the sparse data for a Qobj in operator form
+        # args[n][1] = function callback giving the coefficient
+        #
+        H = H + H_list[n][0] * H_list[n][1](t, psi, args)
+
+    return H * psi
+
+
 # -----------------------------------------------------------------------------
 # Wave function evolution using a ODE solver (unitary quantum evolution) using
 # a constant Hamiltonian.
 #
-def _sesolve_const(H, psi0, tlist, expt_ops, args, opt):
+def _sesolve_const(H, psi0, tlist, e_ops, args, opt, progress_bar):
     """!
     Evolve the wave function using an ODE solver
     """
+    if debug:
+        print(inspect.stack()[0][3])
 
     if not isket(psi0):
         raise TypeError("psi0 must be a ket")
@@ -212,7 +280,7 @@ def _sesolve_const(H, psi0, tlist, expt_ops, args, opt):
     #
     # setup integrator.
     #
-    initial_vector = psi0.full()
+    initial_vector = psi0.full().ravel()
     r = scipy.integrate.ode(cy_ode_rhs)
     L = -1.0j * H
     r.set_f_params(L.data.data, L.data.indices, L.data.indptr)  # cython RHS
@@ -226,7 +294,8 @@ def _sesolve_const(H, psi0, tlist, expt_ops, args, opt):
     #
     # call generic ODE code
     #
-    return _generic_ode_solve(r, psi0, tlist, expt_ops, opt, norm)
+    return _generic_ode_solve(r, psi0, tlist, e_ops, opt,
+                              progress_bar, norm, dims=psi0.dims)
 
 
 #
@@ -236,11 +305,118 @@ def _ode_psi_func(t, psi, H):
     return H * psi
 
 
+# -----------------------------------------------------------------------------
+# A time-dependent disipative master equation on the list-string format for
+# cython compilation
+#
+def _sesolve_list_str_td(H_list, psi0, tlist, e_ops, args, opt,
+                         progress_bar):
+    """
+    Internal function for solving the master equation. See mesolve for usage.
+    """
+
+    if debug:
+        print(inspect.stack()[0][3])
+
+    #
+    # check initial state: must be a density matrix
+    #
+    if not isket(psi0):
+        raise TypeError("The unitary solver requires a ket as initial state")
+
+    #
+    # construct liouvillian
+    #
+    Ldata = []
+    Linds = []
+    Lptrs = []
+    Lcoeff = []
+
+    # loop over all hamiltonian terms, convert to superoperator form and
+    # add the data of sparse matrix representation to h_coeff
+    for h_spec in H_list:
+
+        if isinstance(h_spec, Qobj):
+            h = h_spec
+            h_coeff = "1.0"
+
+        elif isinstance(h_spec, list):
+            h = h_spec[0]
+            h_coeff = h_spec[1]
+
+        else:
+            raise TypeError("Incorrect specification of time-dependent " +
+                            "Hamiltonian (expected string format)")
+
+        L = -1j * h
+
+        Ldata.append(L.data.data)
+        Linds.append(L.data.indices)
+        Lptrs.append(L.data.indptr)
+        Lcoeff.append(h_coeff)
+
+    # the total number of liouvillian terms (hamiltonian terms +
+    # collapse operators)
+    n_L_terms = len(Ldata)
+
+    #
+    # setup ode args string: we expand the list Ldata, Linds and Lptrs into
+    # and explicit list of parameters
+    #
+    string_list = []
+    for k in range(n_L_terms):
+        string_list.append("Ldata[%d], Linds[%d], Lptrs[%d]" % (k, k, k))
+    for name, value in args.items():
+        if isinstance(value, np.ndarray):
+            string_list.append(name)
+        else:
+            string_list.append(str(value))
+    parameter_string = ",".join(string_list)
+
+    #
+    # generate and compile new cython code if necessary
+    #
+    if not opt.rhs_reuse or config.tdfunc is None:
+        if opt.rhs_filename is None:
+            config.tdname = "rhs" + str(os.getpid()) + str(config.cgen_num)
+        else:
+            config.tdname = opt.rhs_filename
+        cgen = Codegen(h_terms=n_L_terms, h_tdterms=Lcoeff, args=args,
+                       config=config)
+        cgen.generate(config.tdname + ".pyx")
+
+        code = compile('from ' + config.tdname + ' import cy_td_ode_rhs',
+                       '<string>', 'exec')
+        exec(code, globals())
+        config.tdfunc = cy_td_ode_rhs
+
+    #
+    # setup integrator
+    #
+    initial_vector = psi0.full().ravel()
+    r = scipy.integrate.ode(config.tdfunc)
+    r.set_integrator('zvode', method=opt.method, order=opt.order,
+                     atol=opt.atol, rtol=opt.rtol, nsteps=opt.nsteps,
+                     first_step=opt.first_step, min_step=opt.min_step,
+                     max_step=opt.max_step)
+    r.set_initial_value(initial_vector, tlist[0])
+    code = compile('r.set_f_params(' + parameter_string + ')',
+                   '<string>', 'exec')
+
+    exec(code, locals(), args)
+
+    #
+    # call generic ODE code
+    #
+    return _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar,
+                              norm, dims=psi0.dims)
+
+
 # -----------------------------------------------------------------------------
 # Wave function evolution using a ODE solver (unitary quantum evolution), for
 # time dependent hamiltonians
 #
-def _sesolve_list_td(H_func, psi0, tlist, expt_ops, args, opt):
+def _sesolve_list_td(H_func, psi0, tlist, e_ops, args, opt, progress_bar):
     """!
     Evolve the wave function using an ODE solver with time-dependent
     Hamiltonian.
@@ -267,7 +443,7 @@ def _sesolve_list_td(H_func, psi0, tlist, expt_ops, args, opt):
                         'length N-1 where N is the number of ' +
                         'Hamiltonian terms.')
     tflag = 1
-    if opt.rhs_reuse and odeconfig.tdfunc is None:
+    if opt.rhs_reuse and config.tdfunc is None:
         print("No previous time-dependent RHS found.")
         print("Generating one for you...")
         rhs_generate(H_func, args)
@@ -281,8 +457,8 @@ def _sesolve_list_td(H_func, psi0, tlist, expt_ops, args, opt):
     # setup ode args string
     string = ""
     for k in range(lenh):
-        string += ("Hdata[" + str(k) + "],Hinds[" + str(k) +
-                   "],Hptrs[" + str(k) + "],")
+        string += ("Hdata[" + str(k) + "], Hinds[" + str(k) +
+                   "], Hptrs[" + str(k) + "],")
 
     if args:
         td_consts = args.items()
@@ -292,24 +468,24 @@ def _sesolve_list_td(H_func, psi0, tlist, expt_ops, args, opt):
                 string += (",")
 
     # run code generator
-    if not opt.rhs_reuse or odeconfig.tdfunc is None:
+    if not opt.rhs_reuse or config.tdfunc is None:
         if opt.rhs_filename is None:
-            odeconfig.tdname = "rhs" + str(odeconfig.cgen_num)
+            config.tdname = "rhs" + str(os.getpid()) + str(config.cgen_num)
         else:
-            odeconfig.tdname = opt.rhs_filename
+            config.tdname = opt.rhs_filename
         cgen = Codegen(h_terms=n_L_terms, h_tdterms=Lcoeff, args=args,
-                       odeconfig=odeconfig)
-        cgen.generate(odeconfig.tdname + ".pyx")
+                       config=config)
+        cgen.generate(config.tdname + ".pyx")
 
-        code = compile('from ' + odeconfig.tdname + ' import cyq_td_ode_rhs',
+        code = compile('from ' + config.tdname + ' import cy_td_ode_rhs',
                        '<string>', 'exec')
         exec(code, globals())
-        odeconfig.tdfunc = cyq_td_ode_rhs
+        config.tdfunc = cy_td_ode_rhs
     #
     # setup integrator
     #
-    initial_vector = psi0.full()
-    r = scipy.integrate.ode(odeconfig.tdfunc)
+    initial_vector = psi0.full().ravel()
+    r = scipy.integrate.ode(config.tdfunc)
     r.set_integrator('zvode', method=opt.method, order=opt.order,
                      atol=opt.atol, rtol=opt.rtol, nsteps=opt.nsteps,
                      first_step=opt.first_step, min_step=opt.min_step,
@@ -321,14 +497,15 @@ def _sesolve_list_td(H_func, psi0, tlist, expt_ops, args, opt):
     #
     # call generic ODE code
     #
-    return _generic_ode_solve(r, psi0, tlist, expt_ops, opt)
+    return _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar,
+                              norm, dims=psi0.dims)
 
 
 # -----------------------------------------------------------------------------
 # Wave function evolution using a ODE solver (unitary quantum evolution), for
 # time dependent hamiltonians
 #
-def _sesolve_func_td(H_func, psi0, tlist, expt_ops, args, opt):
+def _sesolve_func_td(H_func, psi0, tlist, e_ops, args, opt, progress_bar):
     """!
     Evolve the wave function using an ODE solver with time-dependent
     Hamiltonian.
@@ -343,37 +520,63 @@ def _sesolve_func_td(H_func, psi0, tlist, expt_ops, args, opt):
     #
     # setup integrator
     #
-    H_func_and_args = [H_func]
-    for arg in args:
-        if isinstance(arg, Qobj):
-            H_func_and_args.append(arg.data)
+    new_args = None
+
+    if type(args) is dict:
+        new_args = {}
+        for key in args:
+            if isinstance(args[key], Qobj):
+                new_args[key] = args[key].data
+            else:
+                new_args[key] = args[key]
+
+    elif type(args) is list or type(args) is tuple:
+        new_args = []
+        for arg in args:
+            if isinstance(arg, Qobj):
+                new_args.append(arg.data)
+            else:
+                new_args.append(arg)
+
+        if type(args) is tuple:
+            new_args = tuple(new_args)
+    else:
+        if isinstance(args, Qobj):
+            new_args = args.data
         else:
-            H_func_and_args.append(arg)
+            new_args = args
+
+    initial_vector = psi0.full().ravel()
+
+    if not opt.rhs_with_state:
+        r = scipy.integrate.ode(cy_ode_psi_func_td)
+    else:
+        r = scipy.integrate.ode(cy_ode_psi_func_td_with_state)
 
-    initial_vector = psi0.full()
-    r = scipy.integrate.ode(_ode_psi_func_td)
     r.set_integrator('zvode', method=opt.method, order=opt.order,
                      atol=opt.atol, rtol=opt.rtol, nsteps=opt.nsteps,
                      first_step=opt.first_step, min_step=opt.min_step,
                      max_step=opt.max_step)
     r.set_initial_value(initial_vector, tlist[0])
-    r.set_f_params(H_func_and_args)
+    r.set_f_params(H_func, new_args)
 
     #
     # call generic ODE code
     #
-    return _generic_ode_solve(r, psi0, tlist, expt_ops, opt)
+    return _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar, norm,
+                              dims=psi0.dims)
 
 
 #
 # evaluate dpsi(t)/dt for time-dependent hamiltonian
 #
-def _ode_psi_func_td(t, psi, H_func_and_args):
-    H_func = H_func_and_args[0]
-    args = H_func_and_args[1:]
-
+def _ode_psi_func_td(t, psi, H_func, args):
     H = H_func(t, args)
+    return -1j * (H * psi)
 
+
+def _ode_psi_func_td_with_state(t, psi, H_func, args):
+    H = H_func(t, psi, args)
     return -1j * (H * psi)
 
 
@@ -381,7 +584,8 @@ def _ode_psi_func_td(t, psi, H_func_and_args):
 # Solve an ODE which solver parameters already setup (r). Calculate the
 # required expectation values or invoke callback function at each time step.
 #
-def _generic_ode_solve(r, psi0, tlist, expt_ops, opt, state_norm_func=None):
+def _generic_ode_solve(r, psi0, tlist, e_ops, opt, progress_bar,
+                       state_norm_func=None, dims=None):
     """
     Internal function for solving ODEs.
     """
@@ -390,72 +594,76 @@ def _generic_ode_solve(r, psi0, tlist, expt_ops, opt, state_norm_func=None):
     # prepare output array
     #
     n_tsteps = len(tlist)
-    dt = tlist[1] - tlist[0]
-
-    output = Odedata()
-    output.solver = "mesolve"
+    output = Result()
+    output.solver = "sesolve"
     output.times = tlist
 
-    if isinstance(expt_ops, types.FunctionType):
+    if opt.store_states:
+        output.states = []
+
+    if isinstance(e_ops, types.FunctionType):
         n_expt_op = 0
         expt_callback = True
 
-    elif isinstance(expt_ops, list):
+    elif isinstance(e_ops, list):
 
-        n_expt_op = len(expt_ops)
+        n_expt_op = len(e_ops)
         expt_callback = False
 
         if n_expt_op == 0:
+            # fallback on storing states
             output.states = []
+            opt.store_states = True
         else:
             output.expect = []
             output.num_expect = n_expt_op
-            for op in expt_ops:
-                if op.isherm and psi0.isherm:
+            for op in e_ops:
+                if op.isherm:
                     output.expect.append(np.zeros(n_tsteps))
                 else:
                     output.expect.append(np.zeros(n_tsteps, dtype=complex))
-
     else:
         raise TypeError("Expectation parameter must be a list or a function")
 
     #
     # start evolution
     #
-    psi = Qobj(psi0)
+    progress_bar.start(n_tsteps)
+
+    dt = np.diff(tlist)
+    for t_idx, t in enumerate(tlist):
+        progress_bar.update(t_idx)
 
-    t_idx = 0
-    for t in tlist:
         if not r.successful():
             break
 
         if state_norm_func:
-            psi.data = r.y
-            state_norm = state_norm_func(psi.data)
-            psi.data = psi.data / state_norm
-            r.set_initial_value(r.y / state_norm, r.t)
-        else:
-            psi.data = r.y
+            data = r.y / state_norm_func(r.y)
+            r.set_initial_value(data, r.t)
+
+        if opt.store_states:
+            output.states.append(Qobj(r.y, dims=dims))
 
         if expt_callback:
             # use callback method
-            expt_ops(t, Qobj(psi))
-        else:
-            # calculate all the expectation values,
-            # or output state if no operators
-            if n_expt_op == 0:
-                output.states.append(Qobj(psi))  # copy psi/rho
-            else:
-                for m in range(0, n_expt_op):
-                    output.expect[m][t_idx] = expect(expt_ops[m], psi)
+            e_ops(t, Qobj(r.y, dims=psi0.dims))
 
-        r.integrate(r.t + dt)
-        t_idx += 1
+        for m in range(n_expt_op):
+            output.expect[m][t_idx] = cy_expect_psi(e_ops[m].data,
+                                                    r.y, e_ops[m].isherm)
 
-    if not opt.rhs_reuse and odeconfig.tdname is not None:
+        if t_idx < n_tsteps - 1:
+            r.integrate(r.t + dt[t_idx])
+
+    progress_bar.finished()
+
+    if not opt.rhs_reuse and config.tdname is not None:
         try:
-            os.remove(odeconfig.tdname + ".pyx")
+            os.remove(config.tdname + ".pyx")
         except:
             pass
 
+    if opt.store_final_state:
+        output.final_state = Qobj(r.y)
+
     return output
