@@ -5,10 +5,10 @@ def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
     config = Configuration('qutip', parent_package, top_path)
     # Add all QuTiP subpackages here:
-    config.add_subpackage('cyQ')
+    config.add_subpackage('cy')
     if os.environ['FORTRAN_LIBS'] == 'TRUE':
         config.add_subpackage('fortran')
-    #------------------------------
+
     config.make_config_py()
     return config
 
