@@ -1,24 +1,38 @@
-# This file is part of QuTiP.
+# This file is part of QuTiP: Quantum Toolbox in Python.
 #
-#    QuTiP is free software: you can redistribute it and/or modify
-#    it under the terms of the GNU General Public License as published by
-#    the Free Software Foundation, either version 3 of the License, or
-#    (at your option) any later version.
+#    Copyright (c) 2011 and later, Paul D. Nation and Robert J. Johansson.
+#    All rights reserved.
 #
-#    QuTiP is distributed in the hope that it will be useful,
-#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#    GNU General Public License for more details.
+#    Redistribution and use in source and binary forms, with or without
+#    modification, are permitted provided that the following conditions are
+#    met:
 #
-#    You should have received a copy of the GNU General Public License
-#    along with QuTiP.  If not, see <http://www.gnu.org/licenses/>.
+#    1. Redistributions of source code must retain the above copyright notice,
+#       this list of conditions and the following disclaimer.
 #
-# Copyright (C) 2011-2013, Paul D. Nation & Robert J. Johansson
+#    2. Redistributions in binary form must reproduce the above copyright
+#       notice, this list of conditions and the following disclaimer in the
+#       documentation and/or other materials provided with the distribution.
 #
-###########################################################################
+#    3. Neither the name of the QuTiP: Quantum Toolbox in Python nor the names
+#       of its contributors may be used to endorse or promote products derived
+#       from this software without specific prior written permission.
+#
+#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+#    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+#    HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+#    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+#    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+#    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+#    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+#    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+#    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+###############################################################################
 
-from numpy import linspace, sum
-from numpy.testing import assert_, run_module_suite
+from numpy import linspace, sum, abs
+from numpy.testing import assert_, run_module_suite, assert_equal
 from qutip import *
 from scipy.special import laguerre
 from numpy.random import rand
@@ -148,5 +162,32 @@ def test_wigner_compare_methods_ket():
         assert_(sum(W_qutip2) * dx * dy - 1.0 < 1e-8)
 
 
+def test_wigner_fft_comparse_ket():
+    "Wigner: Compare Wigner fft and iterative for rand. ket"
+    N = 20
+    xvec = linspace(-10, 10, 128)
+    for i in range(3):
+        rho = rand_ket(N)
+
+        Wfft, yvec = wigner(rho, xvec, xvec, method='fft')
+        W = wigner(rho, xvec, yvec, method='iterative')
+
+        Wdiff = abs(W - Wfft)
+        assert_equal(sum(abs(Wdiff)) < 1e-7, True)
+
+
+def test_wigner_fft_comparse_dm():
+    "Wigner: Compare Wigner fft and iterative for rand. dm"
+    N = 20
+    xvec = linspace(-10, 10, 128)
+    for i in range(3):
+        rho = rand_dm(N)
+
+        Wfft, yvec = wigner(rho, xvec, xvec, method='fft')
+        W = wigner(rho, xvec, yvec, method='iterative')
+
+        Wdiff = abs(W - Wfft)
+        assert_equal(sum(abs(Wdiff)) < 1e-7, True)
+
 if __name__ == "__main__":
     run_module_suite()
